/**
 * Class to allow posting to LiveJournal (livejournal.com) via a wireless
 * device.
 *
 * Copyright (C) 2002 Brian Enigma <enigma@netninja.com>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MobileLJ.java,v 1.6 2002/04/19 22:12:25 enigma Exp $
 */

import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;

public class MobileLJ 
    extends MIDlet
    implements CommandListener
{
    private static final String ABOUT_TITLE =
        "MobileLJ 1.0";
    private static final String ABOUT_TEXT =
        "Copyright 2002 Brian Enigma <enigma@netninja.com> under the terms of the GPL";
    private static final String LIST_PREFS = "Login Prefs";
    private static final String LIST_POST = "Post";
    private static final String LIST_ABOUT = "About";
    /** Post URL */
    private String postUrl = null;
    /** Post request URL */
    private String requestPath = null;
    private String host = null;
    /** Preferences object */
    private Prefs preferences = null;
    /** Main menu */
    private List mainMenuList;
    /** Select command */
    private Command selectCommand = new Command("Select", Command.OK, 1);
    /** Exit command */
    private Command exitCommand = new Command("Exit", Command.BACK, 1);
    
    protected void startApp()
    {
	boolean appInValidState = true;
        // Get the app settings
        if (postUrl == null)
            postUrl = getAppProperty("postUrl");
        if (requestPath == null)
            requestPath = getAppProperty("requestPath");
        if (host == null)
            host = getAppProperty("host");
        // If the app settings are not there, abort!
        if ((postUrl == null) || (requestPath == null) || (host == null))
	{
            die("No Post URL", "Your Java Application Descriptor seems invalid.  Try installing this app again.");
	    appInValidState = false;
	}
        // Grab prefs from RecordStore
        preferences = new Prefs();
        preferences.load();
        // Build menu
        mainMenuList = new List("Main Menu", List.IMPLICIT, 
            new String[] {LIST_PREFS, LIST_POST, LIST_ABOUT}, null);
        // If no prefs need to be set, skip past the prefs menu item
        modifyList();
        mainMenuList.setCommandListener(this);
	if (appInValidState)
		mainMenuList.addCommand(selectCommand);
        mainMenuList.addCommand(exitCommand);
        Display.getDisplay(this).setCurrent(mainMenuList);
    }

    public void pauseApp() {}
    public void destroyApp(boolean unconditional) {}

    /**
     * Abort with an error dialog.
     */
    private void die(String title, String message)
    {
        Alert alert = new Alert(title, message, null, AlertType.ERROR);
        alert.setTimeout(Alert.FOREVER);
        Display.getDisplay(this).setCurrent(alert, mainMenuList);
    }

    public void commandAction(Command c, Displayable d)
    {
        // Handle main menu options
        if ((c == List.SELECT_COMMAND) || (c == selectCommand))
        {
            String menuItem = mainMenuList.getString(
                mainMenuList.getSelectedIndex());
            if (LIST_PREFS.equals(menuItem))
            {
                // Show preferences menu
                PrefsMenu pm = new PrefsMenu(this, mainMenuList, 
                    preferences);
                pm.show();
            } else if (LIST_POST.equals(menuItem)) {
                PostForm pf = new PostForm(this, mainMenuList);
                pf.setUrl(this.postUrl);
                pf.setRequestPath(this.requestPath);
                pf.setHost(this.host);
                pf.show();
            } else if (LIST_ABOUT.equals(menuItem)) {
                // Show about dialog
                Alert about = new Alert(ABOUT_TITLE, ABOUT_TEXT, 
                    null, AlertType.INFO);
                about.setTimeout(Alert.FOREVER);
                Display.getDisplay(this).setCurrent(about);
            }
        } else if (c == exitCommand) {
            this.notifyDestroyed();
        }
    }

    public void modifyList()
    {
        // Ensure preferences are up to date
        preferences.load();
        // Put post at top of list, if preferences look okay
        mainMenuList.delete(0);
        mainMenuList.delete(0);
        mainMenuList.insert(0, LIST_PREFS, null);
        if (preferences.postworthy())
            mainMenuList.insert(0, LIST_POST, null);
        else
            mainMenuList.insert(1, LIST_POST, null);
        Display.getDisplay(this).setCurrent(mainMenuList);
    }
}


