/**
 * A simple class to hold, save, and load preference values on a J2ME
 * device.  
 *
 * Copyright (C) 2002 Brian Enigma <enigma@netninja.com>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: NetworkPost.java,v 1.6 2002/04/21 00:19:32 enigma Exp $
 */

import java.io.*;
import java.util.Calendar;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import javax.microedition.io.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;

public class NetworkPost
implements Runnable
{
    private Calendar postDate = Calendar.getInstance();
    private String subject = null;
    private String body = null;
    private String url;
    private String path;
    private String host;
    private StringBuffer response;
    private static final long TIMEOUT_SECONDS = 20;
    private static final String END_TOKEN = "[END}";
    private Displayable backForm;
    private Form spinnerForm;
    private Gauge spinner;
    private MIDlet midlet;
	Timer timer = new Timer();

    /** Simple initializer */
    public NetworkPost() {}

    /**
     * x-www-form-urlencode a string.  Shouldn't this be a native library
     * function?!?!????!!!!!
     */
    private String encode(String value)
    {
        StringBuffer result = new StringBuffer();
        int max = value.length();
        for (int i=0; i<max; i++)
        {
            char c = value.charAt(i);
            if (    ((c >= 'a') && (c <= 'z'))
                 || ((c >= 'A') && (c <= 'Z'))
                 || ((c >= '0') && (c <= '9'))
               )
            {
                result.append(c);
            } else if (c == ' ') {
                result.append('+');
            } else {
                result.append('%');
                if (c <= 0x0F) result.append('0');
                result.append(Integer.toString(c, 16));
            }
        }
        return result.toString();
    }
    
    /** 
     * Build POST parameters.  This *could* eat up good amounts of memory.
     * We try to free it up where we can, but...
     */ 
    private String buildParameters()
    {
        StringBuffer result = new StringBuffer();
        Prefs preferences = new Prefs();
        preferences.load();
        result.append("mode=postevent&user=");
        result.append(encode(preferences.getUid()));
        result.append("&password=");
        result.append(encode(preferences.getPwd()));
        // Attempt to free a little memory
        preferences = null;
        result.append("&year=");
        result.append(postDate.get(Calendar.YEAR));
        result.append("&mon=");
        result.append(postDate.get(Calendar.MONTH)+1);
        result.append("&day=");
        result.append(postDate.get(Calendar.DAY_OF_MONTH)+1);
        result.append("&hour=");
        result.append(postDate.get(Calendar.HOUR_OF_DAY));
        result.append("&min=");
        result.append(postDate.get(Calendar.MINUTE));
        result.append("&subject=");
        result.append(encode(subject));
        // Attempt to free a little memory
        subject = null;
        result.append("&event=");
        result.append(encode(body));
        // Attempt to free a little memory
        body = null;
        return result.toString();
    }

    // Runnable interface: start up blocking post call
    public void post()
    {
        spinnerForm = new Form("Posting Entry");
        spinner = new Gauge("Sending", false, 10, 0);
        spinnerForm.append(spinner);
        Display.getDisplay(midlet).setCurrent(spinnerForm);
        // Start timer?
        //timer.schedule(new GaugeSpinner(spinner, 10), 50, 50);
        new Thread(this).start();
    }

    public void run()
    {
        // Perform blocking post call
        postImpl();
        System.gc();
        // Kill timer?
        timer.cancel();
        // Go to previous menu
        //System.out.println(response.toString());
        Alert alert = new Alert("Results", response.toString(), 
            null, AlertType.INFO);
        alert.setTimeout(Alert.FOREVER);
        Display.getDisplay(midlet).setCurrent(alert, this.backForm);
    }

    // Perform the blocking network call to post our entry
    private void postImpl()
    {
        String postData = buildParameters();
        StreamConnection c = null;
        try{
            //System.out.println(this.url);
            c = (StreamConnection)Connector.open(this.url, 
                Connector.READ_WRITE);
            if (c==null) 
                throw new Exception("Unable to open connection");
            OutputStream out = c.openOutputStream();
            if (out == null) 
                throw new Exception("Unable to open output stream");
            out.write("POST ".getBytes());
            out.write(path.getBytes());
            out.write(" HTTP/1.0\r\n".getBytes());
            out.write("Host: ".getBytes());
            out.write(this.host.getBytes());
            out.write("\r\nContent-type: application/x-www-form-urlencoded\r\n".getBytes());
            out.write("Content-length: ".getBytes());
            out.write(Integer.toString(postData.length()).getBytes());
            out.write("\r\n\r\n".getBytes());
            //System.out.println(postData.length());
            //System.out.println(postData);
            out.write(postData.getBytes());
            out.flush();
            // Attempt to free a little memory
            postData = null;
            getServerResponse(c);
            out.close();
            c.close();
        }catch(Exception e){response.append(e.toString());}
        //System.out.println(response.toString());
    }

    private void getServerResponse(StreamConnection c)
    {
        long endDate = new Date().getTime() + TIMEOUT_SECONDS * 100;
        response = new StringBuffer();
        int current = 0;
        try{
            DataInputStream in = new DataInputStream(c.openInputStream());
            while (new Date().getTime() < endDate)
            {
                while (in.available() > 0)
                {
                    current++;
                    response.append((char)in.readByte());
                    spinner.setValue(current % 10);
                }
                if (validResponse())
                    break;
            }
            in.close();
        }catch(IOException e){
            response.append(e.toString());
        }
    }

    // Scan the input stream for a valid response and return
    private boolean validResponse()
    {
        if (response.toString().indexOf(END_TOKEN) != -1)
        {
            parseResponse();
            return true;
        }
        return false;
    }

    // Interpret/remove headers from received data
    private void parseResponse()
    {
        // Check that "200" response was returned, if not return error code
	int pos = response.toString().indexOf(' ');
	int pos2 = response.toString().indexOf('\n');
    pos++;
	if ((pos != -1) && (pos2 != -1) && (pos2 > pos))
	{
	    String httpResponse = response.toString().substring(pos, pos2);
	    if (!httpResponse.startsWith("200"))
	    {
		response = new StringBuffer(httpResponse);
		return;
	    }
	}
        
        // Chop out headers
        pos = response.toString().indexOf("\r\n\r\n");
        if (pos != -1)
            response = new StringBuffer(response.toString().substring(pos+4));
        // Chop out end token, if present
        pos = response.toString().indexOf(END_TOKEN);
        if (pos != -1)
            response = new StringBuffer(response.toString().substring(0,pos));
    }

    /*
    private class GaugeSpinner
    extends TimerTask
    {
	private Gauge spinner;
	private int max;
	private int current = 0;
	private GaugeSpinner() {}
	public GaugeSpinner(Gauge g, int max) {this.spinner=g;}

	// Each time the timer is invoked, increment the spinner
	public void run ()
	{
        //System.out.println("Click");
	    current++;
	    current %= max;
	    spinner.setValue(current);
	}
    }
    */

    // Standard getters
    public Date getDate() {return this.postDate.getTime();}
    public String getSubject() {return this.subject;}
    public String getBody() {return this.body;}
    public String getUrl() {return this.url;}
    public String getHost() {return this.host;}
    public String getRequestPath() {return this.path;}
    public String getResponse() {return this.response.toString();}
    public Displayable getBackForm() {return this.backForm;}
    public MIDlet getMidlet() {return this.midlet;}

    // Standard setters
    public void setDate(Date d) {this.postDate.setTime(d);}
    public void setSubject(String s) {this.subject = s;}
    public void setBody(String s) {this.body = s;}
    public void setUrl(String s) {this.url = s;}
    public void setHost(String s) {this.host = s;}
    public void setBackForm(Displayable d) {this.backForm = d;}
    public void setRequestPath(String s) {this.path = s;}
    public void setMidlet(MIDlet m) {this.midlet = m;}
}
