/**
 * Class to handle the journal entry post form.
 * 
 * Copyright (C) 2002 Brian Enigma <enigma@netninja.com>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: PostForm.java,v 1.8 2002/04/21 00:58:56 enigma Exp $
 */

import java.util.Date;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;

public class PostForm
implements CommandListener
{
    /** MIDlet */
    private MIDlet midlet;
    /** "Back" button form */
    private Displayable backForm;
    /** Post URL */
    private String url;
    private String requestPath;
    private String host;
    /** GUI form **/
    private Form postForm;
    /** Cancel command */
    private Command cancelCommand = new Command("Cancel", Command.CANCEL, 1);
    /** Save command */
    private Command saveCommand = new Command("Post", Command.OK, 1);
    /** Date/Time field */
    private DateField dateField;
    /** Subject Field */
    private TextField subjectText;
    /** Post Field */
    private TextField bodyText;

    /** Cannot construct this way */
    private PostForm() {}

    /** Constructor */
    public PostForm(MIDlet midlet, Displayable backForm)
    {
        this.midlet = midlet;
        this.backForm = backForm;
        dateField = new DateField(null, DateField.DATE_TIME);
        dateField.setDate(new Date());
        subjectText = new TextField("Subject", "", 
            64, TextField.ANY);
        bodyText = new TextField("Entry", "", 
            4000, TextField.ANY);
        postForm = new Form("New Journal Entry");
        postForm.append(dateField);
        postForm.append(subjectText);
        postForm.append(bodyText);
        postForm.addCommand(cancelCommand);
        postForm.addCommand(saveCommand);
        postForm.setCommandListener(this);
    }

    public void show()
    {
        Display.getDisplay(midlet).setCurrent(postForm);
    }

    public void commandAction(Command c, Displayable d)
    {
        if (c == cancelCommand)
        {
            Display.getDisplay(midlet).setCurrent(backForm);
        } else if (c == saveCommand) {
            // TODO: Flesh this out
            NetworkPost np = new NetworkPost();
            np.setBody(bodyText.getString());
            np.setSubject(subjectText.getString());
            np.setDate(dateField.getDate());
            np.setUrl(url);
            np.setHost(host);
            np.setRequestPath(requestPath);
            np.setBackForm(this.backForm);
            np.setMidlet(this.midlet);
            // Attempt to free up references, should allow memory to
            // be freed better when the NetworkPost object drops
            // the reference too.
            bodyText.setString(null);
            subjectText.setString(null);
            np.post();
            /*
            Alert response = new Alert("Response", np.getResponse(),
                null, AlertType.INFO);
            response.setTimeout(Alert.FOREVER);
            Display.getDisplay(midlet).setCurrent(response, backForm);
            */
        }
    }

    // Standard getters
    public String getUrl() {return this.url;}
    public String getHost() {return this.host;}
    public String getRequestPath() {return this.requestPath;}
    // Standard setters
    public void setUrl(String s) {this.url = s;}
    public void setHost(String s) {this.host = s;}
    public void setRequestPath(String s) {this.requestPath = s;}
}
