import javax.microedition.rms.*;

/**
 * A simple class to hold, save, and load preference values on a J2ME
 * device.  
 *
 * Copyright (C) 2002 Brian Enigma <enigma@netninja.com>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Prefs.java,v 1.2 2002/04/16 04:09:09 enigma Exp $
 */
public class Prefs
{
    /** RecordStore name */
    private static final String RECORDSTORE_NAME = "LJPhone";
    /** Record ID in the RecordStore for the UID property */
    private static final int UID_ID = 1;
    /** Record ID in the RecordStore for the PWD property */
    private static final int PWD_ID = 2;

    /** Username */
    private String uid;
    /** Password */
    private String pwd;
   
    /** Simple initializer */
    public Prefs() {initialize();}

    /** 
     * Load preferences from the record store 
     */
    public void load()
    {
        try{
            RecordStore rs = 
                RecordStore.openRecordStore(RECORDSTORE_NAME, false);
            try{
                byte[] buff = rs.getRecord(UID_ID);
                this.uid = new String(buff);
                buff = rs.getRecord(PWD_ID);
                this.pwd = new String(buff);
            }catch(Exception e){initialize();}
            rs.closeRecordStore();
        }catch(Exception e){System.err.println(e.toString()); initialize();}
    }

    /** 
     * Save preferences to the record store 
     */
    public void store()
    {
        try{
            RecordStore.deleteRecordStore(RECORDSTORE_NAME);
            RecordStore rs = 
                RecordStore.openRecordStore(RECORDSTORE_NAME, true);
            rs.addRecord(this.uid.getBytes(), 0, this.uid.length());
            rs.addRecord(this.pwd.getBytes(), 0, this.pwd.length());
            rs.closeRecordStore();
        }catch(Exception e){System.err.println(e.toString()); initialize();}
    }

    /** 
     * Revert to the previously saved data 
     */
    public void revert()
    {
        load();
    }

    /**
     * Return whether enough data is present to allow posting.  In other
     * words before a user can post, the MUST have a user name, password,
     * etc.
     */
    public boolean postworthy()
    {
        return (this.uid.length() > 0) && (this.pwd.length() > 0);
    }

    /** 
     * Initialize the variables 
     */
    private void initialize()
    {
        this.uid = "";
        this.pwd = "";
    }

    // Standard getters
    public String getUid() {return this.uid;}
    public String getPwd() {return this.pwd;}

    // Standard setters
    public void setUid(String s) {this.uid = s;}
    public void setPwd(String s) {this.pwd = s;}
}
