<?
// Script to take the post from a mobile wireless Java device and relay it 
// to LiveJournal
//
// Copyright (C) 2002 Brian Enigma <enigma@netninja.com>
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: mobilelj.php,v 1.5 2002/04/19 08:29:16 enigma Exp $
//
// The use of an intermediate script like this may sound a little weird at 
// first, but this is not only the official suggested way of doing 
// processing like this (if the LJ protocol changes, only this relay server 
// needs to change--otherwise thousands of people will need to download a 
// new cellphone client), but may even be the only way to do this.  The LJ 
// server expects to receive data in a plain HTTP/1.0 socket, but some of 
// the more advanced cellphones will sent it as a HTTP/1.1 chuncked data,
// which causes LJ to barf.  Instead of implementing an entire HTTP/1.0 
// implementation using raw sockets and having to parse the results (which 
// may not even be possible given the memory footprint of smaller phone 
// devices), they will HTTP post using whatever version of the protocol 
// they want to this page, which will make the LJ post, obtain the 
// results, distill the result code(s) to something understandable,
// then return that simple result/error code to the cellphone.

//Define this to take the system offline
//define("OFFLINE_MESSAGE", "The system is temporarily offline");

//Define this for debugging client code (will not actually post to LJ)
//define("NO_REMOTE", "true");

define("SERVERNAME", "www.livejournal.com");
define("SERVERPORT", 80);
define("CLIENTVERSION", "Java-MobileLJ/1.0");
define("END_TOKEN", "[END}");

header("Content-type", "text/plain");

if (defined("OFFLINE_MESSAGE"))
    die(OFFLINE_MESSAGE . END_TOKEN);

// Check that the request was POSTed
if (strcmp($REQUEST_METHOD, "POST") != 0)
    die("MobleLJ:Expected POST data" . END_TOKEN);

// Validate input
$postErrors = badPostData();
if (strlen($postErrors) > 0)
    die("MobileLJ:Missing required field(s):" . $postErrors . END_TOKEN);

if (defined("NO_REMOTE"))
    die("Your post was received but relaying is DISABLED, " .
        "so it was not sent to LJ" . END_TOKEN);

///// ===== ----- First, attempt a login

$postData = "mode=login&clientversion=" . urlEncode(CLIENTVERSION) .
    "&user=" . urlEncode($user) . 
    "&password=" . urlEncode($password);
$postResult = postToServer($postData);
$resultCode = extractError($postResult);
if (strcmp(substr($resultCode, 0, 3), "200") != 0)
    die("Login error, " . $resultCode . END_TOKEN);
$responsePairs = extractPairs($postResult);
if (strcmp($responsePairs["success"], "OK") != 0)
    die("Login error, LJ server reports:" . $responsePairs["errmsg"] . 
    END_TOKEN);

///// ===== ----- Second, post the message

$postData = "mode=postevent" .
    "&user=" . urlEncode($user) . 
    "&password=" . urlEncode($password) . 
    "&year=" . urlEncode($year) .
    "&mon=" . urlEncode($mon) .
    "&day=" . urlEncode($day) .
    "&hour=" . urlEncode($hour) .
    "&min=" . urlEncode($min) .
    "&subject=" . urlEncode($subject) .
    "&event=" . urlEncode($event);
$postResult = postToServer($postData);
$resultCode = extractError($postResult);
if (strcmp(substr($resultCode, 0, 3), "200") != 0)
    die("Post error, " . $resultCode . END_TOKEN);
$responsePairs = extractPairs($postResult);
if (strcmp($responsePairs["success"], "OK") != 0)
    die("Post error, LJ server reports:" . $responsePairs["errmsg"] . 
    END_TOKEN);

print("Successful post!" . END_TOKEN);

// This simply validates the incoming POST fields.  If they are all there
// and fine, false is returned.  If there was a problem, a list of one
// or more problematic fields are returned.
function badPostData()
{
    global $user,$password,$year,$mon,$day,$hour,$min,$subject,$event;

    $result = "";
    if (strlen($user) == 0)     $result .= "username ";
    if (strlen($password) == 0) $result .= "password ";
    if (strlen($year) == 0)     $result .= "year ";
    if (strlen($mon) == 0)      $result .= "month ";
    if (strlen($day) == 0)      $result .= "day ";
    if (strlen($hour) == 0)     $result .= "hour ";
    if (strlen($min) == 0)      $result .= "minute ";
    if (strlen($subject) == 0)  $result .= "subject ";
    if (strlen($event) == 0)    $result .= "event ";
    return $result;
}

// This will post data to the LiveJournal server and return the document
// (including headers) that LJ gives us in response to our post.
function postToServer($postData)
{
    // Adding this header will ensure a failure:
    //"Transfer-encoding: chunked\r\n"

    $result = "";
    $sendData = "POST /interface/flat HTTP/1.0\r\n" .
        "Host: " . SERVERNAME . "\r\n" .  
        "Content-type: application/x-www-form-urlencoded\r\n" .
        "Content-length: " . strlen($postData) . "\r\n\r\n" .
        $postData;

    $handle = fsockopen(SERVERNAME, SERVERPORT, $errorNo, $errorStr, 30);
    if (!handle)
        return "Failed:" . $errorNo . ":" . $errorStr;
    fputs($handle, $sendData);
    //$timeout = $time + 60;
    while (!feof($handle))
    {
        $result .= fgets($handle, 1024);
    }
    fclose($handle);
    return $result;
}

// Given a "HTTP/1.0 200 OK" or "HTTP/1.1 200 OK" string, this will
// return just the error code number and everything up to the first
// newline.  If substr(returnValue, 0, 3)=="200" then everything
// went fine, otherwise we want to display the entire result of this
// function to the user because both the error number and error message
// will be useful.
function extractError($data)
{
    // Get at first space
    $code = strstr($data, " ");
    if (!$code) return "No data returned from LJ server";
    // Chop off at the space
    $code = substr($code, 1);
    $pos = strpos($code, "\n");
    $code = substr($code, 0, $pos-1);
    if (strlen($code) == 0) return "Bad data returned from server";
    return $code;
}

// Take LJ response pairs and parse them into an associative array
function extractPairs($data)
{
    $result = array();
    // Chop out headers
    $data = strstr($data, "\r\n\r\n");
    if (!$data) return $result;
    $data = substr($data, 4);
    $splitString = explode("\n", $data);
    // Grab pairs
    $max = sizeof($splitString);
    for ($i = 0; $i < $max; $i += 2)
    {
        $name = $splitString[$i];
        if ($i < $max)
            $value = $splitString[$i+1];
        else
            $value = "";
        $name = trim($name);
        $value = trim($value);
        if (strlen($name) > 0)
        {
            //print($name . "=" . $value . "<br>");
            $result[$name] = $value;
        }
    }
    return $result;
}

?>
